import { distF, uuidv4 } from './util.js'
import Entity from './entity.js';

class Player extends Entity {
    constructor(you, isPlayer, game = false) {
        super(game);

        this.camera = { x: -this.pos.x, y: -this.pos.y };

        this.rot = 0;
        this.dir = 1;
        this.ticks = 0;

        this.health = 100;

        this.you = you || uuidv4();

        this.isPlayer = isPlayer;

        this.headCount = 0;

        this.type = 'Player';

        this.isMenu = false;

        this.r = 1;

        this.isYou = false;

        this.serverProps = [
            'type', 'camera', 'pos', 'vel', 'rot', 'dir', 'ticks', 'health', 'you', 'isPlayer', 'headCount', 'isMenu', 'r', 'isYou', 'username'
        ];

        this.legalProps = [
            'vel', 'dir', 'camera', 'ticks', 'isMenu', 'r', 'ref'
        ];
    }
    bump() {
        let player = this;

        if (player.ticks < 15) {
            player.dir *= -1;
        }

        player.vel.x *= 0.3;
        player.vel.y *= 0.3;

        player.vel.x += Math.sin(player.rot) * 12;
        player.vel.y -= Math.cos(player.rot) * 12;

        player.ticks = 0;
    }
    handleTick(game) {
    
        let { entities, width, height } = game;

        let ent = this;
        ent.headCount += 1/200;

        ent.headCount = Math.round(ent.headCount * 1000) / 1000;

        if (ent.health <= 0) return;

        ent.pos.x += ent.vel.x;
        ent.pos.y += ent.vel.y;

        ent.pos.x = Math.max(Math.min(ent.pos.x, width / 2), - width / 2);
        ent.pos.y = Math.max(Math.min(ent.pos.y, height / 2), - height / 2);

        ent.bounce(game);

        ent.vel.x *= 0.9;
        ent.vel.y *= 0.9;

        ent.rot += 0.03 * ent.dir;
        ent.rot = ent.rot % (Math.PI * 10);

        ent.camera.x = -ent.pos.x * 0.1 + ent.camera.x * 0.9;
        ent.camera.y = -ent.pos.y * 0.1 + ent.camera.y * 0.9;

        ent.ticks++;

        let velness = Math.sqrt(ent.vel.x ** 2 + ent.vel.y ** 2);

        for (let target of entities) {
            if (target.you == ent.you) continue;

            let dist = distF(ent, target);

            let dp = (Math.sin(ent.rot) * (ent.pos.x - target.pos.x))
                - (Math.cos(ent.rot) * (ent.pos.y - target.pos.y));

            dp /= Math.sqrt(dist + 0.1);

            if (Math.sqrt(dist) < 128 && 1 / dp < -0.2) {
                if (target.type == 'NPC' && velness > 0.2 && ent.ticks >= 15) {
                    ent.isMenu = true;
                    ent.rot = 1.2;
                }

                if (target.type == 'Shooter') {
                    this.health -= 1;
                }

                if (target.immortal) continue;

                let oldHealth = target.health

                let dmg = Math.floor((dp - 0.001) * 10);

                target.health += Math.floor((dp - 0.001) * 10);

                target.vel.x += (target.pos.x - ent.pos.x) * 0.1;
                target.vel.y += (target.pos.y - ent.pos.y) * 0.1;

                if (target.health <= 0 && oldHealth > 0) {
                    console.log(`Player ${target.you} died to a player ${ent.you}`);
                    ent.health -= dmg;
                    if (ent.health > 100) ent.health = 100;
                    ent.headCount += 500;
                }
            }
        }
    }
}

export default Player;